/*
Ver. 1.00 - 2019-06-17

Changelog:
1.00 - 2019-06-17 - initial, based on tx_rw_ferrari_458_wheel_emu_16buttons.ino

This sketch emulates Thrustmaster TMX Pro stock wheel
allowing to connect arduino to TMX Pro wheelbase
and emulate button presses - up to 16 buttons can be connected to arduino (because stock TMX Pro wheel has 12 buttons + 4 for D_Pad)

Thrustmaster Wheelbase cable pinout
1 - Green    - not used
2 - Blue     - GND (ground)
3 - White    - MISO (master in, slave out - to read the data from the wheel)
4 - Orange   - SS (slave select, or PL - parallel load, set it to 0 when you want to read the data)
5 - Red      - CLK (clock impulses)
6 - Black    - +VCC (+3.3 volts! if you have Arduino 5V version - use external +3.3V power or use the wheelbase power!!!)

Arduino UNO pins -> TMX wheelbase cable pins

Arduino GND                 -> TMX Blue wire (2)
Arduino pin 12              -> TMX White wire (3) (data from wheel to base)
Arduino pin 10 + pin 2 (SS) -> TMX Orange wire pin (4) (yes, on Arduino it's wired to two pins. 10 - SS, 2 - INT0)
Arduino pin 13 (SCK)        -> TMX Red wire (5)
Arduino +5V                 -> TMX Black wire (6) (it gives 3.3V, but must be connected into +5V socket on arduino uno side)

Button mappings (we send 8 bytes to wheelbase, only 3 first bytes matter, only 16 bits are used for buttons)

Byte 1 (constant)
  1 - not used
  1 - n/u
  0 - n/u
  1 - n/u
  1 - n/u
  0 - n/u
  0 - n/u
  1 – n/u
  
Byte 2
  1 – Menu (button 8, bottom center)      ->   pin 7 (portD bit7)
  1 – Menu (button 12, bottom right)      ->   pin 6 (portD bit6)
  1 – Gear Up (RB paddle, button 2)       ->   pin 5 (portD bit5)
  1 – B (button 9)                        ->   pin 4 (portD bit4)
  1 – RB (button 11, bottom right)        ->   pin 3 (portD bit3)
  1 – A (button 5)                        ->   pin 11 (portB bit3)
  1 – Y (button 10)                       ->   pin 1 (portD bit1) - if you don't use serial debug (UART TX)
  1 – X (button 3)                        ->   pin 0 (portD bit0) - if you don't use serial debug (UART RX)

Byte 3
  1 – D-Pad Down                          ->   pin 9 (portB bit1)
  1 – View (button 7, bottom center)      ->   pin 8 (portB bit0)
  1 – LB (button 6, bottom left)          ->   pin A5 (portC bit5)
  1 – View (button 4, bottom left)        ->   pin A4 (portC bit4)
  1 – Gear Down (LB paddle, button 1)     ->   pin A3 (portC bit3)
  1 - D-Pad Left                          ->   pin A2 (portC bit2)
  1 - D-Pad Up                            ->   pin A1 (portC bit1)
  1 - D-Pad Right                         ->   pin A0 (portC bit0)
  
Each push button has 2 pins - you connect one to Arduino, another to the Ground :)
Pressed button gives you "0" (grounded signal), released = "1" (pulled-up to +VCC)

Feel free to modify and use for your needs.
This sketch and the documentation above provided "AS IS" under the BSD New license.
http://opensource.org/licenses/BSD-3-Clause

(c) Taras Ivaniukovich (blog@rr-m.org)
http://rr-m.org/blog/

Special thanks to Jake L. for reading his TMX Pro wheel and testing.
*/

//#include "pins_arduino.h" // not necessary in 1.6.3?

byte wheelState [8]; // local push-buttons state saved here
volatile byte pos;

void setup (void) {
  DDRB  |= B00001011; // digital pins 8,9,11 used as inputs with a pull-up to +VCC
  PORTB |= B00001011;
  
  DDRC  |= B00111111; // pins 14-19 (A0 - A5) also used as digital inputs
  PORTC |= B00111111; // pulled-up to +VCC via internal 100k resistors
  
  DDRD  |= B11111011; // digital pins 0,1,3,4,5,6,7 used as inputs
  PORTD |= B11111011; // pulled-up to +VCC via internal 100k resistors
  
  wheelState[0] = B11011001; // TMX Pro wheel first data byte - constant
  wheelState[1] = B11111111; // second data byte - buttons
  wheelState[2] = B11111111; // third data byte - buttons
  wheelState[3] = B11111111; // this and below - not used, but wheelbase reads all 8 bytes...
  wheelState[4] = B11111111;
  wheelState[5] = B11111111;
  wheelState[6] = B11111111;
  wheelState[7] = B11111111;
  
  //Serial.begin(9600);    // Arduino debug console - occupies pins RX (0) and TX (1) on Uno
  pinMode(MISO, OUTPUT); // arduino is a slave device
  SPCR |= _BV(SPE);      // Enables the SPI when 1
  SPCR |= _BV(SPIE);     // Enables the SPI interrupt when 1
  
  // interrupt for SS rising edge. Arduino Uno Pin10 must be connected to Pin2!!!
  attachInterrupt (0, ss_rising, RISING);
  
}

// Interrupt0 (external, pin 2) - prepare to start the transfer
void ss_rising () {
  SPDR = wheelState[0]; // load first byte into SPI data register
  pos = 1;
}

// SPI interrupt routine - this transfers data to wheelbase byte-by-byte
ISR (SPI_STC_vect) {
  SPDR = wheelState[pos++]; // load the next byte to SPI output register and return.
}

void loop() {
  // scan button presses and save that to wheelState array. Data transfer to wheelbase is interrupt-driven - see above.
  //wheelState[0] - always constant for TMX Pro wheel
  wheelState[1] = ((PINB & B00001000) >> 1) | (PIND & B11111011); // take bit 3 from PORTB + the rest from PORTD B11111x11
  wheelState[2] = ((PINB & B00000011) << 6) | (PINC & B00111111); // take bits 0,1 from PORTB + bits 0-5 from PORTC
}
